# pylint: disable=W0212,W0603

import sys
import os
import ctypes
import weakref
import atexit
import platform
from primesense import _openni2 as c_api
from primesense.utils import (inherit_properties, HandleObject, _py_to_ctype_obj, ClosedHandle, InitializationError,
    OpenNIError)


arch = int(platform.architecture()[0].lower().replace("bit", ""))

_default_dll_directories = []
if arch == 32:
    if "OPENNI2_REDIST" in os.environ:
        _default_dll_directories.append(os.environ["OPENNI2_REDIST"])
elif arch == 64:
    if "OPENNI2_REDIST64" in os.environ:
        _default_dll_directories.append(os.environ["OPENNI2_REDIST64"])
    elif "OPENNI2_REDIST" in os.environ:
        _default_dll_directories.append(os.environ["OPENNI2_REDIST"])

_default_dll_directories.append(".")

if sys.platform == "win32":
    _dll_name = "OpenNI2.dll"
elif sys.platform == "darwin":
    _dll_name = "libOpenNI2.dylib"
else:
    _dll_name = "libOpenNI2.so"


SENSOR_IR = c_api.OniSensorType.ONI_SENSOR_IR
SENSOR_COLOR = c_api.OniSensorType.ONI_SENSOR_COLOR
SENSOR_DEPTH = c_api.OniSensorType.ONI_SENSOR_DEPTH

PIXEL_FORMAT_DEPTH_1_MM = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_DEPTH_1_MM
PIXEL_FORMAT_DEPTH_100_UM = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_DEPTH_100_UM
PIXEL_FORMAT_SHIFT_9_2 = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_SHIFT_9_2
PIXEL_FORMAT_SHIFT_9_3 = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_SHIFT_9_3
PIXEL_FORMAT_RGB888 = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_RGB888
PIXEL_FORMAT_YUV422 = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_YUV422
PIXEL_FORMAT_GRAY8 = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_GRAY8
PIXEL_FORMAT_GRAY16 = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_GRAY16
PIXEL_FORMAT_JPEG = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_JPEG
PIXEL_FORMAT_YUYV = c_api.OniPixelFormat.ONI_PIXEL_FORMAT_YUYV

IMAGE_REGISTRATION_DEPTH_TO_COLOR = c_api.OniImageRegistrationMode.ONI_IMAGE_REGISTRATION_DEPTH_TO_COLOR
IMAGE_REGISTRATION_OFF = c_api.OniImageRegistrationMode.ONI_IMAGE_REGISTRATION_OFF


_openni2_initialized = False
loaded_dll_directory = None

def initialize(dll_directories = _default_dll_directories):
    global _openni2_initialized
    global loaded_dll_directory
    if _openni2_initialized:
        return
    if isinstance(dll_directories, str):
        dll_directories = [dll_directories]
    if loaded_dll_directory:
        c_api.oniInitialize(c_api.ONI_API_VERSION)
        _openni2_initialized = True
        return

    found = False
    prev = os.getcwd()
    exceptions = []
    dll_directories = [os.path.normpath(os.path.abspath(d)) for d in dll_directories]

    for dlldir in dll_directories:
        if not os.path.isdir(dlldir):
            exceptions.append((dlldir, "Directory does not exist"))
            continue
        fullpath = os.path.join(dlldir, _dll_name)
        if not os.path.isfile(fullpath):
            exceptions.append((fullpath, "file does not exist"))
            continue
        try:
            os.chdir(dlldir)
            c_api.load_dll(fullpath)
            c_api.oniInitialize(c_api.ONI_API_VERSION)
        except Exception as ex:
            exceptions.append((fullpath, ex))
        else:
            found = True
            loaded_dll_directory = dlldir
            break

    os.chdir(prev)
    if not found:
        raise InitializationError("OpenNI2 could not be loaded:\n    %s" %
            ("\n    ".join("%s: %s" % (dir, ex) for dir, ex in exceptions)),)

    _openni2_initialized = True


def is_initialized():
    return _openni2_initialized

_registered_devices = weakref.WeakSet()
_registered_video_frames = weakref.WeakSet()
_registered_video_streams = weakref.WeakSet()
_registered_recorders = weakref.WeakSet()
_registered_device_listeners = weakref.WeakSet()

def unload():
    global _openni2_initialized
    if not _openni2_initialized:
        return
    for coll in [_registered_video_frames, _registered_recorders, _registered_video_streams, _registered_device_listeners,
            _registered_devices]:
        for hndl in coll:
            hndl.close()
        coll.clear()

    _openni2_initialized = False
    c_api.oniShutdown()

atexit.register(unload)

def get_version():
    return c_api.oniGetVersion()

def wait_for_any_stream(streams, timeout = None):
    if timeout is None:
        timeout = c_api.ONI_TIMEOUT_FOREVER
    else:
        timeout = int(timeout * 1000)  # in msec
    ready_stream_index = ctypes.c_int(-1)
    arr = (c_api.OniStreamHandle * len(streams))()
    for i, s in enumerate(streams):
        arr[i] = s._handle
    try:
        c_api.oniWaitForAnyStream(arr, len(streams), ctypes.byref(ready_stream_index), timeout)
    except OpenNIError as ex:
        if ex.code == c_api.OniStatus.ONI_STATUS_TIME_OUT:
            # timed out
            return None
        else:
            raise
    if ready_stream_index.value >= 0:
        return streams[ready_stream_index.value]
    else:
        return None

VideoMode = c_api.OniVideoMode
DeviceInfo = c_api.OniDeviceInfo

class SensorInfo(object):
    def __init__(self, info):
        self.sensorType = info.sensorType
        self.videoModes = [info.pSupportedVideoModes[i] for i in range(info.numSupportedVideoModes)]
    @classmethod
    def from_stream_handle(cls, handle):
        pinfo = c_api.oniStreamGetSensorInfo(handle)
        if pinfo == 0:
            return None
        return cls(pinfo[0])
    @classmethod
    def from_device_handle(cls, handle, sensor_type):
        pinfo = c_api.oniDeviceGetSensorInfo(handle, sensor_type)
        if not pinfo:
            return None
        return cls(pinfo[0])

class PlaybackSupport(object):
    __slots__ = ["device"]
    def __init__(self, device):
        self.device = weakref.proxy(device)
    def get_speed(self):
        return self.device.get_property(c_api.ONI_DEVICE_PROPERTY_PLAYBACK_SPEED, ctypes.c_float)
    def set_speed(self, speed):
        return self.device.set_property(c_api.ONI_DEVICE_PROPERTY_PLAYBACK_SPEED, speed)
    speed = property(get_speed, set_speed)

    def get_repeat_enabled(self):
        return bool(self.device.get_property(c_api.ONI_DEVICE_PROPERTY_PLAYBACK_REPEAT_ENABLED, c_api.OniBool))
    def set_repeat_enabled(self, enable):
        self.device.set_property(c_api.ONI_DEVICE_PROPERTY_PLAYBACK_REPEAT_ENABLED, enable)
    repeat = property(get_repeat_enabled, set_repeat_enabled)

    def seek(self, stream, frame_index):
        seek = c_api.OniSeek(frameIndex = frame_index, stream = stream._handle)
        self.device.invoke(c_api.ONI_DEVICE_COMMAND_SEEK, ctypes.byref(seek))
    def get_number_of_frames(self, stream):
        return stream.get_number_of_frames()

class Device(HandleObject):
    def __init__(self, uri, mode = None):
        self._orig_uri = uri
        self.uri = uri
        self._mode = mode
        HandleObject.__init__(self, None)
        self._reopen()
        _registered_devices.add(self)
    
    def _reopen(self):
        self.close()
        self._handle = c_api.OniDeviceHandle()
        assert not bool(self._handle)
        if self._mode:
            c_api.oniDeviceOpenEx(self._orig_uri, self._mode, ctypes.byref(self._handle))
        else:
            c_api.oniDeviceOpen(self._orig_uri, ctypes.byref(self._handle))
        assert bool(self._handle), "Handle is NULL after open"
        if self.is_file():
            self.playback = PlaybackSupport(self)
        else:
            self.playback = None
        self._sensor_infos = {}
        self._devinfo = None
        self.get_device_info()

    @classmethod
    def enumerate_uris(cls):
        pdevs = ctypes.POINTER(c_api.OniDeviceInfo)()
        count = ctypes.c_int()
        c_api.oniGetDeviceList(ctypes.byref(pdevs), ctypes.byref(count))
        uris = [pdevs[i].uri for i in range(count.value)]
        c_api.oniReleaseDeviceList(pdevs)
        return uris

    @classmethod
    def open_all(cls):
        return [cls(uri) for uri in cls.enumerate_uris()]

    @classmethod
    def open_any(cls):
        return cls(None)

    @classmethod
    def open_file(cls, filename):
        return cls(filename)

    def _close(self):
        if is_initialized():
            c_api.oniDeviceClose(self._handle)
        self.playback = None

    def get_device_info(self):
        if self._devinfo is None:
            self._devinfo = c_api.OniDeviceInfo()
            c_api.oniDeviceGetInfo(self._handle, ctypes.byref(self._devinfo))
            self.uri = self._devinfo.uri
        return self._devinfo
    device_info = property(get_device_info)

    def get_sensor_info(self, sensor_type):
        if sensor_type in self._sensor_infos:
            return self._sensor_infos[sensor_type]

        info = SensorInfo.from_device_handle(self._handle, sensor_type)
        self._sensor_infos[sensor_type] = info
        return info

    def has_sensor(self, sensor_type):
        return self.get_sensor_info(sensor_type) is not None

    def create_stream(self, sensor_type):
        return VideoStream(self, sensor_type)

    def create_depth_stream(self):
        if not self.has_sensor(SENSOR_DEPTH):
            return None
        return VideoStream(self, SENSOR_DEPTH)

    def create_color_stream(self):
        if not self.has_sensor(SENSOR_COLOR):
            return None
        return VideoStream(self, SENSOR_COLOR)

    def create_ir_stream(self):
        if not self.has_sensor(SENSOR_IR):
            return None
        return VideoStream(self, SENSOR_IR)

    def get_property(self, property_id, rettype):
        ret = rettype()
        size = ctypes.c_int(ctypes.sizeof(ret))
        c_api.oniDeviceGetProperty(self._handle, property_id, ctypes.byref(ret), ctypes.byref(size))
        return ret
    def get_int_property(self, property_id):
        return self.get_property(property_id, ctypes.c_int).value
    def set_property(self, property_id, obj, size = None):
        obj, size = _py_to_ctype_obj(obj)
        if size is None:
            size = ctypes.sizeof(obj)
        c_api.oniDeviceSetProperty(self._handle, property_id, ctypes.byref(obj), size)
    def is_property_supported(self, property_id):
        return bool(c_api.oniDeviceIsPropertySupported(self._handle, property_id))

    def invoke(self, command_id, data, size = None):
        data, size = _py_to_ctype_obj(data)
        if size is None:
            size = ctypes.sizeof(data)
        c_api.oniDeviceInvoke(self._handle, command_id, ctypes.byref(data), size)
		
    def is_command_supported(self, command_id):
        return bool(c_api.oniDeviceIsCommandSupported(self._handle, command_id))

    def is_image_registration_mode_supported(self, mode):
        return bool(c_api.oniDeviceIsImageRegistrationModeSupported(self._handle, mode))
    def get_image_registration_mode(self):
        return self.get_property(c_api.ONI_DEVICE_PROPERTY_IMAGE_REGISTRATION, c_api.OniImageRegistrationMode)
    def set_image_registration_mode(self, mode):
        self.set_property(c_api.ONI_DEVICE_PROPERTY_IMAGE_REGISTRATION, mode)

    def is_file(self):
        return (self.is_property_supported(c_api.ONI_DEVICE_PROPERTY_PLAYBACK_SPEED) and
            self.is_property_supported(c_api.ONI_DEVICE_PROPERTY_PLAYBACK_REPEAT_ENABLED) and
            self.is_command_supported(c_api.ONI_DEVICE_COMMAND_SEEK))

    def get_depth_color_sync_enabled(self):
        return bool(c_api.oniDeviceGetDepthColorSyncEnabled(self._handle))
    def set_depth_color_sync_enabled(self, enable):
        if enable:
            c_api.oniDeviceEnableDepthColorSync(self._handle)
        else:
            c_api.oniDeviceDisableDepthColorSync(self._handle)
    depth_color_sync = property(get_depth_color_sync_enabled, set_depth_color_sync_enabled)

@inherit_properties(c_api.OniFrame, "_frame")
class VideoFrame(HandleObject):
    def __init__(self, pframe):
        self._frame = pframe[0]
        HandleObject.__init__(self, pframe)
        _registered_video_frames.add(self)
    def _close(self):
        if is_initialized():
            c_api.oniFrameRelease(self._handle)
        self._frame = ClosedHandle

    def get_buffer_as(self, ctype):
        return (ctype * int(self.dataSize / ctypes.sizeof(ctype))).from_address(self.data)

    def get_buffer_as_uint8(self):
        return self.get_buffer_as(ctypes.c_uint8)
    def get_buffer_as_uint16(self):
        return self.get_buffer_as(ctypes.c_uint16)
    def get_buffer_as_triplet(self):
        return self.get_buffer_as(ctypes.c_uint8 * 3)


class CameraSettings(object):
    __slots__ = ["stream"]
    def __init__(self, stream):
        self.stream = weakref.proxy(stream)

    def get_auto_exposure(self):
        return bool(self.stream.get_property(c_api.ONI_STREAM_PROPERTY_AUTO_EXPOSURE, c_api.OniBool))
    def set_auto_exposure(self, enabled):
        self.stream.set_property(c_api.ONI_STREAM_PROPERTY_AUTO_EXPOSURE, enabled)
    auto_exposure = property(get_auto_exposure, set_auto_exposure)

    def get_auto_white_balance(self):
        return bool(self.stream.get_property(c_api.ONI_STREAM_PROPERTY_AUTO_WHITE_BALANCE, c_api.OniBool))
    def set_auto_white_balance(self, enabled):
        return self.stream.set_property(c_api.ONI_STREAM_PROPERTY_AUTO_WHITE_BALANCE, enabled)
    auto_white_balance = property(get_auto_white_balance, set_auto_white_balance)

    def get_gain(self):
        try:
            return self.stream.get_property(c_api.ONI_STREAM_PROPERTY_GAIN, ctypes.c_int).value
        except OpenNIError:
            return 100
    def set_gain(self, gain):
        return self.stream.set_property(c_api.ONI_STREAM_PROPERTY_GAIN, gain)
    gain = property(get_gain, set_gain)

    def get_exposure(self):
        try:
            return self.stream.get_property(c_api.ONI_STREAM_PROPERTY_EXPOSURE, ctypes.c_int).value
        except OpenNIError:
            return 0
    def set_exposure(self, exposure):
        return self.stream.set_property(c_api.ONI_STREAM_PROPERTY_EXPOSURE, exposure)
    exposure = property(get_exposure, set_exposure)


class VideoStream(HandleObject):
    def __init__(self, device, sensor_type):
        self.device = device
        self.sensor_type = sensor_type
        self._callbacks = {}
        handle = c_api.OniStreamHandle()
        c_api.oniDeviceCreateStream(self.device._handle, sensor_type, ctypes.byref(handle))
        HandleObject.__init__(self, handle)
        _registered_video_streams.add(self)
        if (self.is_property_supported(c_api.ONI_STREAM_PROPERTY_AUTO_WHITE_BALANCE) and
                self.is_property_supported(c_api.ONI_STREAM_PROPERTY_AUTO_EXPOSURE)):
            self.camera = CameraSettings(self)
        else:
            self.camera = None

    def _close(self):
        if is_initialized():
            self.unregister_all_new_frame_listeners()
            self.stop()
            c_api.oniStreamDestroy(self._handle)
        self.camera = None

    def get_sensor_info(self):
        return SensorInfo.from_stream_handle(self._handle)

    def get_recoder(self, filename, allow_lossy_compression = False):
        rec = Recorder(filename)
        rec.attach(self, allow_lossy_compression)
        return rec

    def start(self):
        c_api.oniStreamStart(self._handle)
    def stop(self):
        c_api.oniStreamStop(self._handle)

    def read_frame(self):
        pframe = ctypes.POINTER(c_api.OniFrame)()
        c_api.oniStreamReadFrame(self._handle, ctypes.byref(pframe))
        return VideoFrame(pframe)

    def register_new_frame_listener(self, callback):
        """callback(stream : VideoStream) -> None"""
        if callback in self._callbacks:
            raise ValueError("Callback %r already registered" % (callback,))
        def adapter(handle, cookie):
            callback(self)

        cb_handle = c_api.OniCallbackHandle()
        cbobj = c_api.OniNewFrameCallback(adapter)
        self._callbacks[callback] = (cb_handle, adapter, cbobj)
        c_api.oniStreamRegisterNewFrameCallback(self._handle, cbobj, None, ctypes.byref(cb_handle))

    def unregister_new_frame_listener(self, callback):
        if callback not in self._callbacks:
            return
        cb_handle, _, _ = self._callbacks.pop(callback)
        c_api.oniStreamUnregisterNewFrameCallback(self._handle, cb_handle)

    def unregister_all_new_frame_listeners(self):
        for cb_handle, _, _ in self._callbacks.values():
            c_api.oniStreamUnregisterNewFrameCallback(self._handle, cb_handle)
        self._callbacks.clear()

    def get_property(self, property_id, rettype):
        ret = rettype()
        size = ctypes.c_int(ctypes.sizeof(ret))
        c_api.oniStreamGetProperty(self._handle, property_id, ctypes.byref(ret), ctypes.byref(size))
        return ret
    def get_int_property(self, property_id):
        return self.get_property(property_id, ctypes.c_int).value
    def set_property(self, property_id, obj, size = None):
        obj, size = _py_to_ctype_obj(obj)
        if size is None:
            size = ctypes.sizeof(obj)
        c_api.oniStreamSetProperty(self._handle, property_id, ctypes.byref(obj), size)
    def is_property_supported(self, property_id):
        return bool(c_api.oniStreamIsPropertySupported(self._handle, property_id))

    def invoke(self, command_id, data, size = None):
        data, size = _py_to_ctype_obj(data)
        if size is None:
            size = ctypes.sizeof(data)
        c_api.oniStreamInvoke(self._handle, command_id, data, size)
    def is_command_supported(self, command_id):
        return bool(c_api.oniStreamIsCommandSupported(self._handle, command_id))

    def get_video_mode(self):
        return self.get_property(c_api.ONI_STREAM_PROPERTY_VIDEO_MODE, c_api.OniVideoMode)
    def set_video_mode(self, video_mode):
        self.set_property(c_api.ONI_STREAM_PROPERTY_VIDEO_MODE, video_mode)
    video_mode = property(get_video_mode, set_video_mode)
    def configure_mode(self, width, height, fps, pixel_format):
        """shortcut for set_video_mode"""
        mode = VideoMode(resolutionX = width, resolutionY = height, fps = fps, pixelFormat = pixel_format)
        self.set_video_mode(mode)

    def get_max_pixel_value(self):
        return self.get_int_property(c_api.ONI_STREAM_PROPERTY_MAX_VALUE)
    def get_min_pixel_value(self):
        return self.get_int_property(c_api.ONI_STREAM_PROPERTY_MIN_VALUE)

    def is_cropping_supported(self):
        return self.is_property_supported(c_api.ONI_STREAM_PROPERTY_CROPPING)
    def get_cropping(self):
        return self.get_property(c_api.ONI_STREAM_PROPERTY_CROPPING, c_api.OniCropping)
    def set_cropping(self, originX, originY, width, height):
        cropping = c_api.OniCropping(enabled = True, originX = originX, originY = originY, width = width, height = height)
        self.set_property(c_api.ONI_STREAM_PROPERTY_CROPPING, cropping)
    cropping = property(get_property, set_property)

    def reset_cropping(self):
        self.set_property(c_api.ONI_STREAM_PROPERTY_CROPPING, c_api.OniCropping(enabled = False))

    def get_mirroring_enabled(self):
        return bool(self.get_property(c_api.ONI_STREAM_PROPERTY_MIRRORING, c_api.OniBool))
    def set_mirroring_enabled(self, enabled):
        self.set_property(c_api.ONI_STREAM_PROPERTY_MIRRORING, enabled)
    mirroring_enabled = property(get_mirroring_enabled, set_mirroring_enabled)

    def get_horizontal_fov(self):
        return self.get_property(c_api.ONI_STREAM_PROPERTY_HORIZONTAL_FOV, ctypes.c_float).value
    def get_vertical_fov(self):
        return self.get_property(c_api.ONI_STREAM_PROPERTY_VERTICAL_FOV, ctypes.c_float).value

    def get_number_of_frames(self):
        return self.get_int_property(c_api.ONI_STREAM_PROPERTY_NUMBER_OF_FRAMES)

    def set_frame_buffers_allocator(self, allocator = None):
        if not allocator:
            c_api.oniStreamSetFrameBuffersAllocator(self._handle, None, None, None)
        else:
            return c_api.oniStreamSetFrameBuffersAllocator(self._handle,
                allocator._allocate_callback, allocator._free_callback, None)

class FrameAllocator(object):
    def __init__(self):
        # keep reference to the methods (they are passed as callbacks)
        self._alloc_callback = c_api.OniFrameAllocBufferCallback(self._allocate_frame_buffer_callback)
        self._free_callback = c_api.OniFrameFreeBufferCallback(self._free_frame_buffer_callback)

    def allocate_frame_buffer(self, size):
        raise NotImplementedError()
    def free_frame_buffer(self, pdata):
        raise NotImplementedError()

    def _allocate_frame_buffer_callback(self, size, _):
        return self.allocate_frame_buffer(size)
    def _free_frame_buffer_callback(self, pdata, _):
        return self.free_frame_buffer(pdata)

class Recorder(HandleObject):
    def __init__(self, filename):
        self.filename = filename
        handle = c_api.OniRecorderHandle()
        c_api.oniCreateRecorder(filename, ctypes.byref(handle))
        HandleObject.__init__(self, handle)
        _registered_recorders.add(self)

    def _close(self):
        if is_initialized():
            c_api.oniRecorderDestroy(ctypes.byref(self._handle))

    def attach(self, stream, allow_lossy_compression = False):
        c_api.oniRecorderAttachStream(self._handle, stream._handle, allow_lossy_compression)
    def start(self):
        c_api.oniRecorderStart(self._handle)
    def stop(self):
        c_api.oniRecorderStop(self._handle)


def convert_world_to_depth(depthStream, worldX, worldY, worldZ):
    """const VideoStream& depthStream, float worldX, float worldY, float worldZ"""
    out_depthX = ctypes.c_float()
    out_depthY = ctypes.c_float()
    out_depthZ = ctypes.c_float()
    c_api.oniCoordinateConverterWorldToDepth(depthStream._handle, worldX, worldY, worldZ,
        ctypes.byref(out_depthX), ctypes.byref(out_depthY), ctypes.byref(out_depthZ))
    return out_depthX.value, out_depthY.value, out_depthZ.value

def convert_depth_to_world(depthStream, depthX, depthY, depthZ):
    """const VideoStream& depthStream, float depthX, float depthY, float depthZ, float* pWorldX, float* pWorldY, float* pWorldZ"""
    out_depthX = ctypes.c_float()
    out_depthY = ctypes.c_float()
    out_depthZ = ctypes.c_float()
    c_api.oniCoordinateConverterDepthToWorld(depthStream._handle, depthX, depthY, depthZ,
        ctypes.byref(out_depthX), ctypes.byref(out_depthY), ctypes.byref(out_depthZ))
    return out_depthX.value, out_depthY.value, out_depthZ.value

def convert_depth_to_color(depthStream, colorStream, depthX, depthY, depthZ):
    """const VideoStream& depthStream, const VideoStream& colorStream, int depthX, int depthY, DepthPixel depthZ, int* pColorX, int* pColorY"""
    colorX = ctypes.c_int()
    colorY = ctypes.c_int()
    c_api.oniCoordinateConverterDepthToColor(depthStream._handle, colorStream._handle, depthX, depthY, depthZ,
        ctypes.byref(colorX), ctypes.byref(colorY))
    return colorX.value, colorY.value

def get_bytes_per_pixel(format):
    c_api.oniFormatBytesPerPixel(format)

class DeviceListener(HandleObject):
    def __init__(self):
        handle = c_api.OniCallbackHandle()
        self._callbacks = c_api.OniDeviceCallbacks(
            deviceConnected = c_api.OniDeviceInfoCallback(self._on_connected),
            deviceDisconnected = c_api.OniDeviceInfoCallback(self._on_disconnected),
            deviceStateChanged = c_api.OniDeviceStateCallback(self._on_state_changed),
        )
        c_api.oniRegisterDeviceCallbacks(self._callbacks, None, ctypes.byref(handle))
        HandleObject.__init__(self, handle)
        _registered_device_listeners.add(self)
        self._connected_uris = set()
        self._disconnected_uris = set()

    def _close(self):
        if is_initialized():
            c_api.oniUnregisterDeviceCallbacks(self._handle)

    def unregister(self):
        self.close()

    def _on_connected(self, pdevinfo, _):
        devinfo = pdevinfo[0]
        self._disconnected_uris.discard(devinfo.uri)
        if devinfo.uri in self._connected_uris:
            return
        self._connected_uris.add(devinfo.uri)
        self.on_connected(devinfo)
    
    def _on_disconnected(self, pdevinfo, _):
        devinfo = pdevinfo[0]
        self._connected_uris.discard(devinfo.uri)
        if devinfo.uri in self._disconnected_uris:
            return
        self._disconnected_uris.add(devinfo.uri)
        self.on_disconnected(devinfo)
    
    def _on_state_changed(self, pdevinfo, state, _):
        self.on_state_changed(pdevinfo[0], state)

    def on_connected(self, devinfo):
        """Implement me"""
        pass
    def on_disconnected(self, devinfo):
        """Implement me"""
        pass
    def on_state_changed(self, devinfo, state):
        """Implement me"""
        pass

def get_log_filename():
    buf = ctypes.create_string_buffer(1024)
    try:
        c_api.oniGetLogFileName(buf, ctypes.sizeof(buf))
    except OpenNIError:
        # not logging to file
        return None
    else:
        return buf.value

def configure_logging(directory = None, severity = None, console = None):
    """
    directory: directory in which log files will be stored
    severity: 0 - Verbose; 1 - Info; 2 - Warning; 3 - Error. Default - None
    console: whether to print to the console (boolean) 
    """
    if directory is not None:
        c_api.oniSetLogOutputFolder(directory)
        c_api.oniSetLogFileOutput(True)
    else:
        c_api.oniSetLogFileOutput(False)

    if severity is not None:
        c_api.oniSetLogMinSeverity(severity)
    
    if console is not None:
        c_api.oniSetLogConsoleOutput(bool(console))





# pylint: disable=W0212,W0603

import sys
import os
import weakref
import atexit
import ctypes
import platform
from primesense import _nite2 as c_api
from primesense import openni2
from primesense.utils import inherit_properties, ClosedHandle, HandleObject, InitializationError


arch = int(platform.architecture()[0].lower().replace("bit", ""))

_default_dll_directories = []
if arch == 32:
    if "NITE2_REDIST" in os.environ:
        _default_dll_directories.append(os.environ["NITE2_REDIST"])
elif arch == 64:
    if "NITE2_REDIST64" in os.environ:
        _default_dll_directories.append(os.environ["NITE2_REDIST64"])
    elif "NITE2_REDIST" in os.environ:
        _default_dll_directories.append(os.environ["NITE2_REDIST"])

_default_dll_directories.append(".")

if sys.platform == "win32":
    _dll_name = "NiTE2.dll"
elif sys.platform == "darwin":
    _dll_name = "libNiTE2.dylib"
else:
    _dll_name = "libNiTE2.so"


_nite2_initialized = False
loaded_dll_directory = None
def initialize(dll_directories = _default_dll_directories):
    global _nite2_initialized
    global loaded_dll_directory
    if _nite2_initialized:
        return
    if isinstance(dll_directories, str):
        dll_directories = [dll_directories]
        
    if not openni2.is_initialized():
        openni2.initialize()

    if loaded_dll_directory:
        c_api.niteInitialize()
        _nite2_initialized = True
        return
    
    found = False
    prev = os.getcwd()
    exceptions = []
    dll_directories = [os.path.normpath(os.path.abspath(d)) for d in dll_directories]
    
    for dlldir in dll_directories:
        if not os.path.isdir(dlldir):
            exceptions.append((dlldir, "Directory does not exist"))
            continue
        fullpath = os.path.join(dlldir, _dll_name)
        if not os.path.isfile(fullpath):
            exceptions.append((fullpath, "file does not exist"))
            continue
        try:
            os.chdir(dlldir)
            c_api.load_dll(fullpath)
            c_api.niteInitialize()
        except Exception as ex:
            exceptions.append((fullpath, ex))
        else:
            found = True
            loaded_dll_directory = dlldir
            break
    
    os.chdir(prev)
    if not found:
        raise InitializationError("NiTE2 could not be loaded:\n    %s" % 
            ("\n    ".join("%s: %s" % (dir, ex) for dir, ex in exceptions)),)

    _nite2_initialized = True

def is_initialized():
    return _nite2_initialized

_registered_user_trackers = weakref.WeakSet()
_registered_user_tracker_frames = weakref.WeakSet() 
_registered_hand_trackers = weakref.WeakSet()
_registered_hand_tracker_frames = weakref.WeakSet()
_registered_user_tracker_listeners = weakref.WeakSet()
_registered_hand_tracker_listeners = weakref.WeakSet()

def unload():
    global _nite2_initialized
    if not _nite2_initialized:
        return
    for coll in [_registered_user_tracker_frames, _registered_hand_tracker_frames, _registered_hand_trackers, 
            _registered_user_trackers, _registered_user_tracker_listeners, _registered_hand_tracker_listeners]:
        for hndl in coll:
            hndl.close()
        coll.clear()
    
    _nite2_initialized = False
    c_api.niteShutdown()

atexit.register(unload)

def get_version():
    return c_api.niteGetVersion()

Point3f = c_api.NitePoint3f
Plane = c_api.NitePlane
Quaternion = c_api.NiteQuaternion
BoundingBox = c_api.NiteBoundingBox
UserId = c_api.NiteUserId
HandId = c_api.NiteHandId
UserMap = c_api.NiteUserMap
SkeletonJoint = c_api.NiteSkeletonJoint

@inherit_properties(c_api.NitePoseData, "_posedata")
class PoseData(object):
    __slots__ = ["_posedata"]
    def __init__(self, posedata):
        self._posedata = posedata
    def is_held(self):
        return (self.state & c_api.NitePoseState.NITE_POSE_STATE_IN_POSE) != 0
    def is_entered(self):
        return (self.state & c_api.NitePoseState.NITE_POSE_STATE_ENTER) != 0
    def is_exited(self):
        return (self.state & c_api.NitePoseState.NITE_POSE_STATE_EXIT) != 0

@inherit_properties(c_api.NiteSkeleton, "_skeleton")
class Skeleton(object):
    __slots__ = ["_skeleton"]
    def __init__(self, skeleton):
        self._skeleton = skeleton
    def get_joint(self, jointtype):
        return self.joints[jointtype]

@inherit_properties(c_api.NiteUserData, "_userdata")
class UserData(object):
    __slots__ = ["_userdata"]
    def __init__(self, userdata):
        self._userdata = userdata
    def is_new(self):
        return (self.state & c_api.NiteUserState.NITE_USER_STATE_NEW) != 0
    def is_visible(self):
        return (self.state & c_api.NiteUserState.NITE_USER_STATE_VISIBLE) != 0;
    def is_lost(self):
        return (self.state & c_api.NiteUserState.NITE_USER_STATE_LOST) != 0;
    def get_pose(self, posetype):
        return PoseData(self.poses[posetype])

@inherit_properties(c_api.NiteUserTrackerFrame, "_frame")
class UserTrackerFrame(HandleObject):
    __slots__ = ["_frame", "_user_tracker_handle", "_depth_frame", "users", "users_by_id", "__weakref__"]
    def __init__(self, pframe, user_tracker_handle):
        self._frame = pframe[0]
        self._user_tracker_handle = user_tracker_handle
        self._depth_frame = None
        c_api.niteUserTrackerFrameAddRef(user_tracker_handle, pframe)
        HandleObject.__init__(self, pframe)
        self.users = []
        self.users_by_id = {}
        for i in range(self.userCount):
            u = UserData(self.pUser[i])
            self.users.append(u)
            self.users_by_id[u.id] = u
        _registered_user_tracker_frames.add(self)
    
    def _close(self):
        if is_initialized():
            c_api.niteUserTrackerFrameRelease(self._user_tracker_handle, self._handle)
        self._frame = ClosedHandle
        self._user_tracker_handle = ClosedHandle
        del self.users[:]
    
    def get_depth_frame(self):
        if self._depth_frame is None:
            self._depth_frame = openni2.VideoFrame(self.pDepthFrame)
        return self._depth_frame


class _NiteDevStruct(ctypes.Structure):
    _fields_ = [
        ("pPlaybackControl", ctypes.c_void_p),
        ("device", openni2.c_api.OniDeviceHandle),
    ]

class UserTracker(HandleObject):
    def __init__(self, device):
        handle = c_api.NiteUserTrackerHandle()
        if not device:
            c_api.niteInitializeUserTracker(ctypes.byref(handle))
        else:
            self._devstruct = _NiteDevStruct()
            self._devstruct.device = device._handle
            c_api.niteInitializeUserTrackerByDevice(ctypes.byref(self._devstruct), ctypes.byref(handle))
        HandleObject.__init__(self, handle)
        _registered_user_trackers.add(self)
        
    @classmethod
    def open_any(cls):
        return UserTracker(None)

    def _close(self):
        if is_initialized():
            c_api.niteShutdownUserTracker(self._handle)

    def read_frame(self):
        pnf = ctypes.POINTER(c_api.NiteUserTrackerFrame)()
        c_api.niteReadUserTrackerFrame(self._handle, ctypes.byref(pnf))
        return UserTrackerFrame(pnf, self._handle)

    def set_skeleton_smoothing_factor(self, factor):
        return c_api.niteSetSkeletonSmoothing(self._handle, factor)
    def get_skeleton_smoothing_factor(self):
        factor = ctypes.c_float()
        c_api.niteGetSkeletonSmoothing(self._handle, ctypes.byref(factor))
        return factor.value
    skeleton_smoothing_factor = property(get_skeleton_smoothing_factor, set_skeleton_smoothing_factor)

    def start_skeleton_tracking(self, userid):
        c_api.niteStartSkeletonTracking(self._handle, userid)
    def stop_skeleton_tracking(self, userid):
        c_api.niteStopSkeletonTracking(self._handle, userid)
    
    def is_tracking(self, userid):
        c_api.niteIsSkeletonTracking(self._handle, userid)

    def start_pose_detection(self, userid, posetype):
        c_api.niteStartPoseDetection(self._handle, userid, posetype)
    def stop_pose_detection(self, userid, posetype):
        c_api.niteStopPoseDetection(self._handle, userid, posetype)
    def stop_all_pose_detection(self, userid):
        c_api.niteStopAllPoseDetection(self._handle, userid)

    def convert_joint_coordinates_to_depth(self,  x, y, z):
        outX = ctypes.c_float()
        outY = ctypes.c_float()
        c_api.niteConvertJointCoordinatesToDepth(self._handle, x, y, z, ctypes.byref(outX), ctypes.byref(outY))
        return (outX.value, outY.value)
    
    def convert_depth_coordinates_to_joint(self, x, y, z):
        outX = ctypes.c_float()
        outY = ctypes.c_float()
        c_api.niteConvertDepthCoordinatesToJoint(self._handle, x, y, z, ctypes.byref(outX), ctypes.byref(outY))
        return (outX.value, outY.value)


@inherit_properties(c_api.NiteGestureData, "_gesture")
class GestureData(object):
    def __init__(self, gesture):
        self._gesture = gesture

    def is_complete(self):
        return (self.state & c_api.NiteGestureState.NITE_GESTURE_STATE_COMPLETED) != 0
    def is_in_progress(self):
        return (self.state & c_api.NiteGestureState.NITE_GESTURE_STATE_IN_PROGRESS) != 0


@inherit_properties(c_api.NiteHandData, "_handdata")
class HandData(object):
    def __init__(self, handdata):
        self._handdata = handdata
    
    def is_new(self):
        return (self.state & c_api.NiteHandState.NITE_HAND_STATE_NEW) != 0
    def is_lost(self):
        return self.state == c_api.NiteHandState.NITE_HAND_STATE_LOST
    def is_tracking(self):
        return (self.state & c_api.NiteHandState.NITE_HAND_STATE_TRACKED) != 0
    def is_touching_fov(self):
        return (self.state & c_api.NiteHandState.NITE_HAND_STATE_TOUCHING_FOV) != 0

@inherit_properties(c_api.NiteHandTrackerFrame, "_frame")
class HandTrackerFrame(HandleObject):
    def __init__(self, hand_tracker_handle, pframe):
        self._hand_tracker_handle = hand_tracker_handle
        self._frame = pframe[0]
        c_api.niteHandTrackerFrameAddRef(hand_tracker_handle, pframe)
        HandleObject.__init__(self, pframe)
        self._depth_frame = None
        self._hands = None
        self._gestures = None
        _registered_hand_tracker_frames.add(self)
    
    def _close(self):
        if is_initialized():
            c_api.niteHandTrackerFrameRelease(self._hand_tracker_handle, self._handle)

    @property
    def depth_frame(self):
        if self._depth_frame is None:
            self._depth_frame = openni2.VideoFrame(self._frame.pDepthFrame)
        return self._depth_frame

    @property
    def hands(self):
        if self._hands is None:
            self._hands = [self._frame.pHands[i] for i in range(self._frame.handCount)]
        return self._hands
    
    @property
    def gestures(self):
        if self._gestures is None:
            self._gestures = [self._frame.pGestures[i] for i in range(self._frame.gestureCount)]
        return self._gestures

class HandTracker(HandleObject):
    def __init__(self, device):
        self.device = device
        handle = c_api.NiteHandTrackerHandle()
        if not device:
            c_api.niteInitializeHandTracker(ctypes.byref(handle))
        else:
            self._devstruct = _NiteDevStruct()
            self._devstruct.device = device._handle
            c_api.niteInitializeHandTrackerByDevice(ctypes.byref(self._devstruct), ctypes.byref(handle))
        HandleObject.__init__(self, handle)
        _registered_hand_trackers.add(self)
    
    @classmethod
    def open_any(cls):
        return cls(None)
    
    def _close(self):
        if is_initialized():
            c_api.niteShutdownHandTracker(self._handle)

    def read_frame(self):
        pfrm = ctypes.POINTER(c_api.NiteHandTrackerFrame)()
        c_api.niteReadHandTrackerFrame(self._handle, ctypes.byref(pfrm))
        return HandTrackerFrame(self._handle, pfrm)

    def set_smoothing_factor(self, factor):
        c_api.niteSetHandSmoothingFactor(self._handle, factor)
    def get_smoothing_factor(self):
        factor = ctypes.c_float()
        c_api.niteGetHandSmoothingFactor(self._handle, ctypes.byref(factor))
        return factor.value
    smoothing_factor = property(get_smoothing_factor, set_smoothing_factor)

    def start_hand_tracking(self, *position):
        new_hand_id = HandId()
        if len(position) == 3:
            position = Point3f(*position)
        elif len(position) == 1:
            position = position[0]
        else:
            raise TypeError("Either Point3f or three values required")
        c_api.niteStartHandTracking(self._handle, ctypes.byref(position), ctypes.byref(new_hand_id))
        return new_hand_id
    def stop_hand_tracking(self, handid):
        c_api.niteStopHandTracking(self._handle, handid)

    def start_gesture_detection(self, gesture_type):
        c_api.niteStartGestureDetection(self._handle, gesture_type)
    def stop_gesture_detection(self, gesture_type):
        c_api.niteStopGestureDetection(self._handle, gesture_type)

    def convert_hand_coordinates_to_depth(self, x, y, z):
        outX = ctypes.c_float()
        outY = ctypes.c_float()
        c_api.niteConvertHandCoordinatesToDepth(self._handle, x, y, z, ctypes.byref(outX), ctypes.byref(outY))
        return outX.value, outY.value
    
    def convert_depth_coordinates_to_hand(self, x, y, z):
        outX = ctypes.c_float()
        outY = ctypes.c_float()
        c_api.niteConvertDepthCoordinatesToHand(self._handle, x, y, z, ctypes.byref(outX), ctypes.byref(outY))
        return outX.value, outY.value
    
    def stop_all_hand_tracking(self):
        c_api.niteStopAllHandTracking(self._handle)
    
    def stop_all_gesture_detection(self):
        c_api.niteStopAllGestureDetection(self._handle)


class UserTrackerListener(HandleObject):
    def __init__(self, user_tracker):
        self.user_tracker = user_tracker
        self._callbacks = c_api.NiteUserTrackerCallbacks(
            readyForNextFrame = c_api.OniGeneralCallback(self._on_ready_for_next_frame))
        handle = ctypes.pointer(self._callbacks)
        c_api.niteRegisterUserTrackerCallbacks(self.user_tracker._handle, handle, None)
        HandleObject.__init__(self, handle)
        _registered_user_tracker_listeners.add(self)
    
    def unregister(self):
        self.close()
    
    def _close(self):
        if is_initialized():
            c_api.niteUnregisterUserTrackerCallbacks(self.user_tracker._handle, self._handle)
            self.user_tracker = None
    
    def _on_ready_for_next_frame(self, _):
        self.on_ready_for_next_frame()
    def on_ready_for_next_frame(self):
        """Implement me"""
        pass


class HandTrackerListener(HandleObject):
    def __init__(self, hand_tracker):
        self.hand_tracker = hand_tracker
        self._callbacks = c_api.NiteHandTrackerCallbacks(
            readyForNextFrame = c_api.OniGeneralCallback(self._on_ready_for_next_frame))
        handle = ctypes.pointer(self._callbacks)
        c_api.niteRegisterHandTrackerCallbacks(self.hand_tracker._handle, handle, None)
        HandleObject.__init__(self, handle)
        _registered_hand_tracker_listeners.add(self)
    
    def _close(self):
        if is_initialized():
            c_api.niteUnregisterHandTrackerCallbacks(self.hand_tracker._handle, self._handle)
            self.hand_tracker = None
    
    def _on_ready_for_next_frame(self, _):
        self.on_ready_for_next_frame()
    def on_ready_for_next_frame(self):
        """Implement me"""
        pass





import ctypes


class InitializationError(Exception):
    pass

class OpenNIError(Exception):
    def __init__(self, code, message, logfile):
        self.code = code
        self.logfile = logfile
        Exception.__init__(self, code, message, logfile)

class NiteError(Exception):
    def __init__(self, code):
        Exception.__init__(self, code)

def inherit_properties(struct, attrname):
    def deco(cls):
        for name, _ in struct._fields_:
            def getter(self, name = name):
                return getattr(getattr(self, attrname), name)
            def setter(self, value, name = name):
                return setattr(getattr(self, attrname), name, value)
            setattr(cls, name, property(getter, setter))
        return cls
    return deco

class ClosedHandleError(Exception):
    pass
class ClosedHandle(object):
    def __getattr__(self, name):
        raise ClosedHandleError("Invalid handle")
    def __bool__(self):
        return False
    __nonzero__ = __bool__
ClosedHandle = ClosedHandle()

class HandleObject(object):
    __slots__ = ["_handle"]
    def __init__(self, handle):
        self._handle = handle
    def __del__(self):
        self.close()
    def __enter__(self):
        return self
    def __exit__(self, t, v, tb):
        self.close()
    def __bool__(self):
        return hasattr(self, "_handle") and bool(self._handle)
    __nonzero__ = __bool__
    def close(self):
        if hasattr(self, "_handle") and self._handle:
            self._close()
            self._handle = ClosedHandle
    def _close(self):
        raise NotImplementedError()

def _py_to_ctype_obj(obj):
    size = None
    if isinstance(obj, (int, bool)):
        obj = ctypes.c_int(obj)
    elif isinstance(obj, float):
        obj = ctypes.c_float(obj)
    elif isinstance(obj, str):
        obj = ctypes.create_string_buffer(obj)
        size = len(obj)
    return obj, size


class CEnumMeta(type(ctypes.c_int)):
    def __new__(cls, name, bases, namespace):
        cls2 = type(ctypes.c_int).__new__(cls, name, bases, namespace)
        if namespace.get("__module__") != __name__:
            namespace["_values_"].clear()
            for name in namespace["_names_"].keys():
                if name.startswith("_"):
                    continue
                setattr(cls2, name, cls2(namespace[name]))
                namespace["_names_"][name] = namespace[name]
                namespace["_values_"][namespace[name]] = name
        return cls2

def with_meta(meta, base = object):
    return meta("NewBase", (base,), {"__module__" : __name__})

class CEnum(with_meta(CEnumMeta, ctypes.c_int)):
    _names_ = {}
    _values_ = {}
    __slots__ = []
    
    def __repr__(self):
        name = self._values_.get(self.value)
        if name is None:
            return "%s(%r)" % (self.__class__.__name__, self.val)
        else:
            return "%s.%s" % (self.__class__.__name__, name)
    @classmethod
    def from_param(cls, obj):
        return int(obj)
    @classmethod
    def from_name(cls, name):
        return cls._names_[name]
    @classmethod
    def from_value(cls, val):
        return getattr(self, cls._values_[val])

    def __int__(self):
        return int(self.value)
    def __index__(self):
        return int(self)
    def __eq__(self, other):
        return int(self) == int(other)
    def __ne__(self, other):
        return int(self) != int(other)
    def __gt__(self, other):
        return int(self) > int(other)
    def __ge__(self, other):
        return int(self) >= int(other)
    def __lt__(self, other):
        return int(self) < int(other)
    def __le__(self, other):
        return int(self) <= int(other)
    def __hash__(self):
        return hash(int(self))


class DLLNotLoaded(Exception):
    pass

class UnloadedDLL(object):
    __slots__ = []
    def __bool__(self):
        return False
    __nonzero__ = __bool__
    def __call__(self, *args, **kwargs):
        raise DLLNotLoaded("DLL is not loaded")
    def __getattr__(self, name):
        raise DLLNotLoaded("DLL is not loaded")

UnloadedDLL = UnloadedDLL()





# Auto-generated file; do not edit directly
# Tue Nov 12 13:55:35 2013
import sys

import ctypes
from primesense.utils import CEnum, UnloadedDLL

TRUE = 1
FALSE = 0
ONI_MAX_STR = 256
ONI_MAX_SENSORS = 10
NITE_JOINT_COUNT = 15
NITE_POSE_COUNT = 2
NITE_VERSION_MAJOR = 2
NITE_VERSION_MINOR = 3
NITE_VERSION_MAINTENANCE = 0
NITE_VERSION_BUILD = 16
NITE_VERSION = ( NITE_VERSION_MAJOR * 100000000 + NITE_VERSION_MINOR * 1000000 + NITE_VERSION_MAINTENANCE * 10000 + NITE_VERSION_BUILD )

def _get_calling_conv(*args):
    if sys.platform == 'win32':
        return ctypes.WINFUNCTYPE(*args)
    else:
        return ctypes.CFUNCTYPE(*args)

from _openni2 import OniStatus
from _openni2 import OniSensorType
from _openni2 import OniPixelFormat
from _openni2 import OniDeviceState
from _openni2 import OniImageRegistrationMode
class _anon_enum_5(CEnum):
    _names_ = {'ONI_TIMEOUT_NONE': 0, 'ONI_TIMEOUT_FOREVER': -1}
    _values_ = {0: 'ONI_TIMEOUT_NONE', -1: 'ONI_TIMEOUT_FOREVER'}
    ONI_TIMEOUT_NONE = 0
    ONI_TIMEOUT_FOREVER = -1

ONI_TIMEOUT_NONE = _anon_enum_5.ONI_TIMEOUT_NONE
ONI_TIMEOUT_FOREVER = _anon_enum_5.ONI_TIMEOUT_FOREVER

from _openni2 import OniCallbackHandleImpl
from _openni2 import OniVersion
from _openni2 import OniVideoMode
from _openni2 import OniSensorInfo
from _openni2 import OniDeviceInfo
class _OniDevice(ctypes.Structure):

    def __repr__(self):
        return '_OniDevice()' % ()

class _OniStream(ctypes.Structure):

    def __repr__(self):
        return '_OniStream()' % ()

class _OniRecorder(ctypes.Structure):

    def __repr__(self):
        return '_OniRecorder()' % ()

from _openni2 import OniFrame
from _openni2 import OniDeviceCallbacks
from _openni2 import OniCropping
from _openni2 import OniRGB888Pixel
from _openni2 import OniYUV422DoublePixel
from _openni2 import OniSeek
class NiteJointType(CEnum):
    _names_ = {'NITE_JOINT_RIGHT_ELBOW': 5, 'NITE_JOINT_LEFT_ELBOW': 4, 'NITE_JOINT_RIGHT_KNEE': 12, 'NITE_JOINT_RIGHT_HAND': 7, 'NITE_JOINT_RIGHT_SHOULDER': 3, 'NITE_JOINT_HEAD': 0, 'NITE_JOINT_LEFT_HAND': 6, 'NITE_JOINT_LEFT_SHOULDER': 2, 'NITE_JOINT_LEFT_KNEE': 11, 'NITE_JOINT_TORSO': 8, 'NITE_JOINT_NECK': 1, 'NITE_JOINT_LEFT_HIP': 9, 'NITE_JOINT_RIGHT_HIP': 10, 'NITE_JOINT_LEFT_FOOT': 13, 'NITE_JOINT_RIGHT_FOOT': 14}
    _values_ = {0: 'NITE_JOINT_HEAD', 1: 'NITE_JOINT_NECK', 2: 'NITE_JOINT_LEFT_SHOULDER', 3: 'NITE_JOINT_RIGHT_SHOULDER', 4: 'NITE_JOINT_LEFT_ELBOW', 5: 'NITE_JOINT_RIGHT_ELBOW', 6: 'NITE_JOINT_LEFT_HAND', 7: 'NITE_JOINT_RIGHT_HAND', 8: 'NITE_JOINT_TORSO', 9: 'NITE_JOINT_LEFT_HIP', 10: 'NITE_JOINT_RIGHT_HIP', 11: 'NITE_JOINT_LEFT_KNEE', 12: 'NITE_JOINT_RIGHT_KNEE', 13: 'NITE_JOINT_LEFT_FOOT', 14: 'NITE_JOINT_RIGHT_FOOT'}
    NITE_JOINT_HEAD = 0
    NITE_JOINT_NECK = 1
    NITE_JOINT_LEFT_SHOULDER = 2
    NITE_JOINT_RIGHT_SHOULDER = 3
    NITE_JOINT_LEFT_ELBOW = 4
    NITE_JOINT_RIGHT_ELBOW = 5
    NITE_JOINT_LEFT_HAND = 6
    NITE_JOINT_RIGHT_HAND = 7
    NITE_JOINT_TORSO = 8
    NITE_JOINT_LEFT_HIP = 9
    NITE_JOINT_RIGHT_HIP = 10
    NITE_JOINT_LEFT_KNEE = 11
    NITE_JOINT_RIGHT_KNEE = 12
    NITE_JOINT_LEFT_FOOT = 13
    NITE_JOINT_RIGHT_FOOT = 14

class NiteSkeletonState(CEnum):
    _names_ = {'NITE_SKELETON_CALIBRATION_ERROR_HEAD': 5, 'NITE_SKELETON_CALIBRATION_ERROR_NOT_IN_POSE': 3, 'NITE_SKELETON_TRACKED': 2, 'NITE_SKELETON_CALIBRATION_ERROR_LEGS': 6, 'NITE_SKELETON_CALIBRATION_ERROR_HANDS': 4, 'NITE_SKELETON_CALIBRATING': 1, 'NITE_SKELETON_CALIBRATION_ERROR_TORSO': 7, 'NITE_SKELETON_NONE': 0}
    _values_ = {0: 'NITE_SKELETON_NONE', 1: 'NITE_SKELETON_CALIBRATING', 2: 'NITE_SKELETON_TRACKED', 3: 'NITE_SKELETON_CALIBRATION_ERROR_NOT_IN_POSE', 4: 'NITE_SKELETON_CALIBRATION_ERROR_HANDS', 5: 'NITE_SKELETON_CALIBRATION_ERROR_HEAD', 6: 'NITE_SKELETON_CALIBRATION_ERROR_LEGS', 7: 'NITE_SKELETON_CALIBRATION_ERROR_TORSO'}
    NITE_SKELETON_NONE = 0
    NITE_SKELETON_CALIBRATING = 1
    NITE_SKELETON_TRACKED = 2
    NITE_SKELETON_CALIBRATION_ERROR_NOT_IN_POSE = 3
    NITE_SKELETON_CALIBRATION_ERROR_HANDS = 4
    NITE_SKELETON_CALIBRATION_ERROR_HEAD = 5
    NITE_SKELETON_CALIBRATION_ERROR_LEGS = 6
    NITE_SKELETON_CALIBRATION_ERROR_TORSO = 7

class NiteUserState(CEnum):
    _names_ = {'NITE_USER_STATE_VISIBLE': 1, 'NITE_USER_STATE_NEW': 2, 'NITE_USER_STATE_LOST': 4}
    _values_ = {1: 'NITE_USER_STATE_VISIBLE', 2: 'NITE_USER_STATE_NEW', 4: 'NITE_USER_STATE_LOST'}
    NITE_USER_STATE_VISIBLE = 1
    NITE_USER_STATE_NEW = 2
    NITE_USER_STATE_LOST = 4

class NiteStatus(CEnum):
    _names_ = {'NITE_STATUS_OUT_OF_FLOW': 3, 'NITE_STATUS_ERROR': 1, 'NITE_STATUS_BAD_USER_ID': 2, 'NITE_STATUS_OK': 0}
    _values_ = {0: 'NITE_STATUS_OK', 1: 'NITE_STATUS_ERROR', 2: 'NITE_STATUS_BAD_USER_ID', 3: 'NITE_STATUS_OUT_OF_FLOW'}
    NITE_STATUS_OK = 0
    NITE_STATUS_ERROR = 1
    NITE_STATUS_BAD_USER_ID = 2
    NITE_STATUS_OUT_OF_FLOW = 3

class NitePoseType(CEnum):
    _names_ = {'NITE_POSE_CROSSED_HANDS': 1, 'NITE_POSE_PSI': 0}
    _values_ = {0: 'NITE_POSE_PSI', 1: 'NITE_POSE_CROSSED_HANDS'}
    NITE_POSE_PSI = 0
    NITE_POSE_CROSSED_HANDS = 1

class NitePoseState(CEnum):
    _names_ = {'NITE_POSE_STATE_IN_POSE': 2, 'NITE_POSE_STATE_ENTER': 4, 'NITE_POSE_STATE_DETECTING': 1, 'NITE_POSE_STATE_EXIT': 8}
    _values_ = {8: 'NITE_POSE_STATE_EXIT', 1: 'NITE_POSE_STATE_DETECTING', 2: 'NITE_POSE_STATE_IN_POSE', 4: 'NITE_POSE_STATE_ENTER'}
    NITE_POSE_STATE_DETECTING = 1
    NITE_POSE_STATE_IN_POSE = 2
    NITE_POSE_STATE_ENTER = 4
    NITE_POSE_STATE_EXIT = 8

class NiteGestureType(CEnum):
    _names_ = {'NITE_GESTURE_HAND_RAISE': 2, 'NITE_GESTURE_CLICK': 1, 'NITE_GESTURE_WAVE': 0}
    _values_ = {0: 'NITE_GESTURE_WAVE', 1: 'NITE_GESTURE_CLICK', 2: 'NITE_GESTURE_HAND_RAISE'}
    NITE_GESTURE_WAVE = 0
    NITE_GESTURE_CLICK = 1
    NITE_GESTURE_HAND_RAISE = 2

class NiteGestureState(CEnum):
    _names_ = {'NITE_GESTURE_STATE_IN_PROGRESS': 2, 'NITE_GESTURE_STATE_COMPLETED': 4, 'NITE_GESTURE_STATE_NEW': 1}
    _values_ = {1: 'NITE_GESTURE_STATE_NEW', 2: 'NITE_GESTURE_STATE_IN_PROGRESS', 4: 'NITE_GESTURE_STATE_COMPLETED'}
    NITE_GESTURE_STATE_NEW = 1
    NITE_GESTURE_STATE_IN_PROGRESS = 2
    NITE_GESTURE_STATE_COMPLETED = 4

class NiteHandState(CEnum):
    _names_ = {'NITE_HAND_STATE_NEW': 1, 'NITE_HAND_STATE_TRACKED': 2, 'NITE_HAND_STATE_LOST': 0, 'NITE_HAND_STATE_TOUCHING_FOV': 4}
    _values_ = {0: 'NITE_HAND_STATE_LOST', 1: 'NITE_HAND_STATE_NEW', 2: 'NITE_HAND_STATE_TRACKED', 4: 'NITE_HAND_STATE_TOUCHING_FOV'}
    NITE_HAND_STATE_LOST = 0
    NITE_HAND_STATE_NEW = 1
    NITE_HAND_STATE_TRACKED = 2
    NITE_HAND_STATE_TOUCHING_FOV = 4

class NiteUserTracker(ctypes.Structure):

    def __repr__(self):
        return 'NiteUserTracker()' % ()

class NitePoint3f(ctypes.Structure):
    x = 'ctypes.c_float'
    y = 'ctypes.c_float'
    z = 'ctypes.c_float'

    def __repr__(self):
        return 'NitePoint3f(x = %r, y = %r, z = %r)' % (self.x, self.y, self.z)

class NiteQuaternion(ctypes.Structure):
    x = 'ctypes.c_float'
    y = 'ctypes.c_float'
    z = 'ctypes.c_float'
    w = 'ctypes.c_float'

    def __repr__(self):
        return 'NiteQuaternion(x = %r, y = %r, z = %r, w = %r)' % (self.x, self.y, self.z, self.w)

class NiteSkeletonJoint(ctypes.Structure):
    jointType = 'NiteJointType'
    position = 'NitePoint3f'
    positionConfidence = 'ctypes.c_float'
    orientation = 'NiteQuaternion'
    orientationConfidence = 'ctypes.c_float'

    def __repr__(self):
        return 'NiteSkeletonJoint(jointType = %r, position = %r, positionConfidence = %r, orientation = %r, orientationConfidence = %r)' % (self.jointType, self.position, self.positionConfidence, self.orientation, self.orientationConfidence)

class NiteBoundingBox(ctypes.Structure):
    min = 'NitePoint3f'
    max = 'NitePoint3f'

    def __repr__(self):
        return 'NiteBoundingBox(min = %r, max = %r)' % (self.min, self.max)

class NitePoseData(ctypes.Structure):
    type = 'NitePoseType'
    state = 'ctypes.c_int'

    def __repr__(self):
        return 'NitePoseData(type = %r, state = %r)' % (self.type, self.state)

class NiteSkeleton(ctypes.Structure):
    joints = '(NiteSkeletonJoint * 15)'
    state = 'NiteSkeletonState'
    internalState = 'ctypes.c_int'

    def __repr__(self):
        return 'NiteSkeleton(joints = %r, state = %r, internalState = %r)' % (self.joints, self.state, self.internalState)

class NiteUserData(ctypes.Structure):
    id = 'NiteUserId'
    boundingBox = 'NiteBoundingBox'
    centerOfMass = 'NitePoint3f'
    state = 'ctypes.c_int'
    skeleton = 'NiteSkeleton'
    poses = '(NitePoseData * 2)'

    def __repr__(self):
        return 'NiteUserData(id = %r, boundingBox = %r, centerOfMass = %r, state = %r, skeleton = %r, poses = %r)' % (self.id, self.boundingBox, self.centerOfMass, self.state, self.skeleton, self.poses)

class NiteUserMap(ctypes.Structure):
    pixels = 'ctypes.POINTER(NiteUserId)'
    width = 'ctypes.c_int'
    height = 'ctypes.c_int'
    stride = 'ctypes.c_int'

    def __repr__(self):
        return 'NiteUserMap(pixels = %r, width = %r, height = %r, stride = %r)' % (self.pixels, self.width, self.height, self.stride)

class NitePlane(ctypes.Structure):
    point = 'NitePoint3f'
    normal = 'NitePoint3f'

    def __repr__(self):
        return 'NitePlane(point = %r, normal = %r)' % (self.point, self.normal)

class NiteUserTrackerFrame(ctypes.Structure):
    userCount = 'ctypes.c_int'
    pUser = 'ctypes.POINTER(NiteUserData)'
    userMap = 'NiteUserMap'
    pDepthFrame = 'ctypes.POINTER(OniFrame)'
    timestamp = 'ctypes.c_ulonglong'
    frameIndex = 'ctypes.c_int'
    floorConfidence = 'ctypes.c_float'
    floor = 'NitePlane'

    def __repr__(self):
        return 'NiteUserTrackerFrame(userCount = %r, pUser = %r, userMap = %r, pDepthFrame = %r, timestamp = %r, frameIndex = %r, floorConfidence = %r, floor = %r)' % (self.userCount, self.pUser, self.userMap, self.pDepthFrame, self.timestamp, self.frameIndex, self.floorConfidence, self.floor)

class NiteUserTrackerCallbacks(ctypes.Structure):
    readyForNextFrame = 'OniGeneralCallback'

    def __repr__(self):
        return 'NiteUserTrackerCallbacks(readyForNextFrame = %r)' % (self.readyForNextFrame)

class NiteHandData(ctypes.Structure):
    id = 'NiteHandId'
    position = 'NitePoint3f'
    state = 'ctypes.c_int'

    def __repr__(self):
        return 'NiteHandData(id = %r, position = %r, state = %r)' % (self.id, self.position, self.state)

class NiteGestureData(ctypes.Structure):
    type = 'NiteGestureType'
    currentPosition = 'NitePoint3f'
    state = 'ctypes.c_int'

    def __repr__(self):
        return 'NiteGestureData(type = %r, currentPosition = %r, state = %r)' % (self.type, self.currentPosition, self.state)

class NiteHandTrackerFrame(ctypes.Structure):
    handCount = 'ctypes.c_int'
    pHands = 'ctypes.POINTER(NiteHandData)'
    gestureCount = 'ctypes.c_int'
    pGestures = 'ctypes.POINTER(NiteGestureData)'
    pDepthFrame = 'ctypes.POINTER(OniFrame)'
    timestamp = 'ctypes.c_ulonglong'
    frameIndex = 'ctypes.c_int'

    def __repr__(self):
        return 'NiteHandTrackerFrame(handCount = %r, pHands = %r, gestureCount = %r, pGestures = %r, pDepthFrame = %r, timestamp = %r, frameIndex = %r)' % (self.handCount, self.pHands, self.gestureCount, self.pGestures, self.pDepthFrame, self.timestamp, self.frameIndex)

class NiteHandTrackerCallbacks(ctypes.Structure):
    readyForNextFrame = 'OniGeneralCallback'

    def __repr__(self):
        return 'NiteHandTrackerCallbacks(readyForNextFrame = %r)' % (self.readyForNextFrame)

class NiteVersion(ctypes.Structure):
    major = 'ctypes.c_int'
    minor = 'ctypes.c_int'
    maintenance = 'ctypes.c_int'
    build = 'ctypes.c_int'

    def __repr__(self):
        return 'NiteVersion(major = %r, minor = %r, maintenance = %r, build = %r)' % (self.major, self.minor, self.maintenance, self.build)

class NiteHandTracker(ctypes.Structure):

    def __repr__(self):
        return 'NiteHandTracker()' % ()

OniBool = ctypes.c_int
OniCallbackHandle = ctypes.POINTER(OniCallbackHandleImpl)
OniHardwareVersion = ctypes.c_int
OniDeviceHandle = ctypes.POINTER(_OniDevice)
OniStreamHandle = ctypes.POINTER(_OniStream)
OniRecorderHandle = ctypes.POINTER(_OniRecorder)
OniNewFrameCallback = _get_calling_conv(None, OniStreamHandle, ctypes.c_void_p)
OniGeneralCallback = _get_calling_conv(None, ctypes.c_void_p)
OniDeviceInfoCallback = _get_calling_conv(None, ctypes.POINTER(OniDeviceInfo), ctypes.c_void_p)
OniDeviceStateCallback = _get_calling_conv(None, ctypes.POINTER(OniDeviceInfo), OniDeviceState, ctypes.c_void_p)
OniFrameAllocBufferCallback = _get_calling_conv(ctypes.c_void_p, ctypes.c_int, ctypes.c_void_p)
OniFrameFreeBufferCallback = _get_calling_conv(None, ctypes.c_void_p, ctypes.c_void_p)
OniDepthPixel = ctypes.c_ushort
OniGrayscale16Pixel = ctypes.c_ushort
OniGrayscale8Pixel = ctypes.c_ubyte
NiteUserId = ctypes.c_short
NiteUserTrackerHandle = ctypes.POINTER(NiteUserTracker)
NiteHandId = ctypes.c_short
NiteHandTrackerHandle = ctypes.POINTER(NiteHandTracker)

_OniDevice._fields_ = [
]

_OniStream._fields_ = [
]

_OniRecorder._fields_ = [
]

NiteUserTracker._fields_ = [
]

NitePoint3f._fields_ = [
    ('x', ctypes.c_float),
    ('y', ctypes.c_float),
    ('z', ctypes.c_float),
]

NiteQuaternion._fields_ = [
    ('x', ctypes.c_float),
    ('y', ctypes.c_float),
    ('z', ctypes.c_float),
    ('w', ctypes.c_float),
]

NiteSkeletonJoint._fields_ = [
    ('jointType', NiteJointType),
    ('position', NitePoint3f),
    ('positionConfidence', ctypes.c_float),
    ('orientation', NiteQuaternion),
    ('orientationConfidence', ctypes.c_float),
]

NiteBoundingBox._fields_ = [
    ('min', NitePoint3f),
    ('max', NitePoint3f),
]

NitePoseData._fields_ = [
    ('type', NitePoseType),
    ('state', ctypes.c_int),
]

NiteSkeleton._fields_ = [
    ('joints', (NiteSkeletonJoint * 15)),
    ('state', NiteSkeletonState),
    ('internalState', ctypes.c_int),
]

NiteUserData._fields_ = [
    ('id', NiteUserId),
    ('boundingBox', NiteBoundingBox),
    ('centerOfMass', NitePoint3f),
    ('state', ctypes.c_int),
    ('skeleton', NiteSkeleton),
    ('poses', (NitePoseData * 2)),
]

NiteUserMap._fields_ = [
    ('pixels', ctypes.POINTER(NiteUserId)),
    ('width', ctypes.c_int),
    ('height', ctypes.c_int),
    ('stride', ctypes.c_int),
]

NitePlane._fields_ = [
    ('point', NitePoint3f),
    ('normal', NitePoint3f),
]

NiteUserTrackerFrame._fields_ = [
    ('userCount', ctypes.c_int),
    ('pUser', ctypes.POINTER(NiteUserData)),
    ('userMap', NiteUserMap),
    ('pDepthFrame', ctypes.POINTER(OniFrame)),
    ('timestamp', ctypes.c_ulonglong),
    ('frameIndex', ctypes.c_int),
    ('floorConfidence', ctypes.c_float),
    ('floor', NitePlane),
]

NiteUserTrackerCallbacks._fields_ = [
    ('readyForNextFrame', OniGeneralCallback),
]

NiteHandData._fields_ = [
    ('id', NiteHandId),
    ('position', NitePoint3f),
    ('state', ctypes.c_int),
]

NiteGestureData._fields_ = [
    ('type', NiteGestureType),
    ('currentPosition', NitePoint3f),
    ('state', ctypes.c_int),
]

NiteHandTrackerFrame._fields_ = [
    ('handCount', ctypes.c_int),
    ('pHands', ctypes.POINTER(NiteHandData)),
    ('gestureCount', ctypes.c_int),
    ('pGestures', ctypes.POINTER(NiteGestureData)),
    ('pDepthFrame', ctypes.POINTER(OniFrame)),
    ('timestamp', ctypes.c_ulonglong),
    ('frameIndex', ctypes.c_int),
]

NiteHandTrackerCallbacks._fields_ = [
    ('readyForNextFrame', OniGeneralCallback),
]

NiteVersion._fields_ = [
    ('major', ctypes.c_int),
    ('minor', ctypes.c_int),
    ('maintenance', ctypes.c_int),
    ('build', ctypes.c_int),
]

NiteHandTracker._fields_ = [
]

_the_dll = UnloadedDLL
_niteInitialize = UnloadedDLL
_niteShutdown = UnloadedDLL
_niteGetVersion = UnloadedDLL
_niteShutdownUserTracker = UnloadedDLL
_niteStartSkeletonTracking = UnloadedDLL
_niteStopSkeletonTracking = UnloadedDLL
_niteIsSkeletonTracking = UnloadedDLL
_niteSetSkeletonSmoothing = UnloadedDLL
_niteGetSkeletonSmoothing = UnloadedDLL
_niteStartPoseDetection = UnloadedDLL
_niteStopPoseDetection = UnloadedDLL
_niteStopAllPoseDetection = UnloadedDLL
_niteRegisterUserTrackerCallbacks = UnloadedDLL
_niteUnregisterUserTrackerCallbacks = UnloadedDLL
_niteReadUserTrackerFrame = UnloadedDLL
_niteUserTrackerFrameAddRef = UnloadedDLL
_niteUserTrackerFrameRelease = UnloadedDLL
_niteDumpUserTrackerCalibrationDataToFile = UnloadedDLL
_niteShutdownHandTracker = UnloadedDLL
_niteStartHandTracking = UnloadedDLL
_niteStopHandTracking = UnloadedDLL
_niteStopAllHandTracking = UnloadedDLL
_niteSetHandSmoothingFactor = UnloadedDLL
_niteGetHandSmoothingFactor = UnloadedDLL
_niteRegisterHandTrackerCallbacks = UnloadedDLL
_niteUnregisterHandTrackerCallbacks = UnloadedDLL
_niteReadHandTrackerFrame = UnloadedDLL
_niteHandTrackerFrameAddRef = UnloadedDLL
_niteHandTrackerFrameRelease = UnloadedDLL
_niteStartGestureDetection = UnloadedDLL
_niteStopGestureDetection = UnloadedDLL
_niteStopAllGestureDetection = UnloadedDLL
_niteConvertJointCoordinatesToDepth = UnloadedDLL
_niteConvertDepthCoordinatesToJoint = UnloadedDLL
_niteConvertHandCoordinatesToDepth = UnloadedDLL
_niteConvertDepthCoordinatesToHand = UnloadedDLL
_niteInitializeUserTracker = UnloadedDLL
_niteInitializeUserTrackerByDevice = UnloadedDLL
_niteInitializeHandTracker = UnloadedDLL
_niteInitializeHandTrackerByDevice = UnloadedDLL

def load_dll(dllname):
    global _the_dll
    if _the_dll:
        raise ValueError('DLL already loaded')
    dll = ctypes.CDLL(dllname)

    global _niteInitialize
    _niteInitialize = dll.niteInitialize
    _niteInitialize.restype  = NiteStatus
    _niteInitialize.argtypes = []

    global _niteShutdown
    _niteShutdown = dll.niteShutdown
    _niteShutdown.restype  = None
    _niteShutdown.argtypes = []

    global _niteGetVersion
    _niteGetVersion = dll.niteGetVersion
    _niteGetVersion.restype  = NiteVersion
    _niteGetVersion.argtypes = []

    global _niteShutdownUserTracker
    _niteShutdownUserTracker = dll.niteShutdownUserTracker
    _niteShutdownUserTracker.restype  = NiteStatus
    _niteShutdownUserTracker.argtypes = [NiteUserTrackerHandle]

    global _niteStartSkeletonTracking
    _niteStartSkeletonTracking = dll.niteStartSkeletonTracking
    _niteStartSkeletonTracking.restype  = NiteStatus
    _niteStartSkeletonTracking.argtypes = [NiteUserTrackerHandle, NiteUserId]

    global _niteStopSkeletonTracking
    _niteStopSkeletonTracking = dll.niteStopSkeletonTracking
    _niteStopSkeletonTracking.restype  = None
    _niteStopSkeletonTracking.argtypes = [NiteUserTrackerHandle, NiteUserId]

    global _niteIsSkeletonTracking
    _niteIsSkeletonTracking = dll.niteIsSkeletonTracking
    _niteIsSkeletonTracking.restype  = ctypes.c_bool
    _niteIsSkeletonTracking.argtypes = [NiteUserTrackerHandle, NiteUserId]

    global _niteSetSkeletonSmoothing
    _niteSetSkeletonSmoothing = dll.niteSetSkeletonSmoothing
    _niteSetSkeletonSmoothing.restype  = NiteStatus
    _niteSetSkeletonSmoothing.argtypes = [NiteUserTrackerHandle, ctypes.c_float]

    global _niteGetSkeletonSmoothing
    _niteGetSkeletonSmoothing = dll.niteGetSkeletonSmoothing
    _niteGetSkeletonSmoothing.restype  = NiteStatus
    _niteGetSkeletonSmoothing.argtypes = [NiteUserTrackerHandle, ctypes.POINTER(ctypes.c_float)]

    global _niteStartPoseDetection
    _niteStartPoseDetection = dll.niteStartPoseDetection
    _niteStartPoseDetection.restype  = NiteStatus
    _niteStartPoseDetection.argtypes = [NiteUserTrackerHandle, NiteUserId, NitePoseType]

    global _niteStopPoseDetection
    _niteStopPoseDetection = dll.niteStopPoseDetection
    _niteStopPoseDetection.restype  = None
    _niteStopPoseDetection.argtypes = [NiteUserTrackerHandle, NiteUserId, NitePoseType]

    global _niteStopAllPoseDetection
    _niteStopAllPoseDetection = dll.niteStopAllPoseDetection
    _niteStopAllPoseDetection.restype  = None
    _niteStopAllPoseDetection.argtypes = [NiteUserTrackerHandle, NiteUserId]

    global _niteRegisterUserTrackerCallbacks
    _niteRegisterUserTrackerCallbacks = dll.niteRegisterUserTrackerCallbacks
    _niteRegisterUserTrackerCallbacks.restype  = NiteStatus
    _niteRegisterUserTrackerCallbacks.argtypes = [NiteUserTrackerHandle, ctypes.POINTER(NiteUserTrackerCallbacks), ctypes.c_void_p]

    global _niteUnregisterUserTrackerCallbacks
    _niteUnregisterUserTrackerCallbacks = dll.niteUnregisterUserTrackerCallbacks
    _niteUnregisterUserTrackerCallbacks.restype  = None
    _niteUnregisterUserTrackerCallbacks.argtypes = [NiteUserTrackerHandle, ctypes.POINTER(NiteUserTrackerCallbacks)]

    global _niteReadUserTrackerFrame
    _niteReadUserTrackerFrame = dll.niteReadUserTrackerFrame
    _niteReadUserTrackerFrame.restype  = NiteStatus
    _niteReadUserTrackerFrame.argtypes = [NiteUserTrackerHandle, ctypes.POINTER(ctypes.POINTER(NiteUserTrackerFrame))]

    global _niteUserTrackerFrameAddRef
    _niteUserTrackerFrameAddRef = dll.niteUserTrackerFrameAddRef
    _niteUserTrackerFrameAddRef.restype  = NiteStatus
    _niteUserTrackerFrameAddRef.argtypes = [NiteUserTrackerHandle, ctypes.POINTER(NiteUserTrackerFrame)]

    global _niteUserTrackerFrameRelease
    _niteUserTrackerFrameRelease = dll.niteUserTrackerFrameRelease
    _niteUserTrackerFrameRelease.restype  = NiteStatus
    _niteUserTrackerFrameRelease.argtypes = [NiteUserTrackerHandle, ctypes.POINTER(NiteUserTrackerFrame)]

    global _niteDumpUserTrackerCalibrationDataToFile
    _niteDumpUserTrackerCalibrationDataToFile = dll.niteDumpUserTrackerCalibrationDataToFile
    _niteDumpUserTrackerCalibrationDataToFile.restype  = NiteStatus
    _niteDumpUserTrackerCalibrationDataToFile.argtypes = [NiteUserTrackerHandle, NiteUserId, ctypes.c_char_p]

    global _niteShutdownHandTracker
    _niteShutdownHandTracker = dll.niteShutdownHandTracker
    _niteShutdownHandTracker.restype  = NiteStatus
    _niteShutdownHandTracker.argtypes = [NiteHandTrackerHandle]

    global _niteStartHandTracking
    _niteStartHandTracking = dll.niteStartHandTracking
    _niteStartHandTracking.restype  = NiteStatus
    _niteStartHandTracking.argtypes = [NiteHandTrackerHandle, ctypes.POINTER(NitePoint3f), ctypes.POINTER(NiteHandId)]

    global _niteStopHandTracking
    _niteStopHandTracking = dll.niteStopHandTracking
    _niteStopHandTracking.restype  = None
    _niteStopHandTracking.argtypes = [NiteHandTrackerHandle, NiteHandId]

    global _niteStopAllHandTracking
    _niteStopAllHandTracking = dll.niteStopAllHandTracking
    _niteStopAllHandTracking.restype  = None
    _niteStopAllHandTracking.argtypes = [NiteHandTrackerHandle]

    global _niteSetHandSmoothingFactor
    _niteSetHandSmoothingFactor = dll.niteSetHandSmoothingFactor
    _niteSetHandSmoothingFactor.restype  = NiteStatus
    _niteSetHandSmoothingFactor.argtypes = [NiteHandTrackerHandle, ctypes.c_float]

    global _niteGetHandSmoothingFactor
    _niteGetHandSmoothingFactor = dll.niteGetHandSmoothingFactor
    _niteGetHandSmoothingFactor.restype  = NiteStatus
    _niteGetHandSmoothingFactor.argtypes = [NiteHandTrackerHandle, ctypes.POINTER(ctypes.c_float)]

    global _niteRegisterHandTrackerCallbacks
    _niteRegisterHandTrackerCallbacks = dll.niteRegisterHandTrackerCallbacks
    _niteRegisterHandTrackerCallbacks.restype  = NiteStatus
    _niteRegisterHandTrackerCallbacks.argtypes = [NiteHandTrackerHandle, ctypes.POINTER(NiteHandTrackerCallbacks), ctypes.c_void_p]

    global _niteUnregisterHandTrackerCallbacks
    _niteUnregisterHandTrackerCallbacks = dll.niteUnregisterHandTrackerCallbacks
    _niteUnregisterHandTrackerCallbacks.restype  = None
    _niteUnregisterHandTrackerCallbacks.argtypes = [NiteHandTrackerHandle, ctypes.POINTER(NiteHandTrackerCallbacks)]

    global _niteReadHandTrackerFrame
    _niteReadHandTrackerFrame = dll.niteReadHandTrackerFrame
    _niteReadHandTrackerFrame.restype  = NiteStatus
    _niteReadHandTrackerFrame.argtypes = [NiteHandTrackerHandle, ctypes.POINTER(ctypes.POINTER(NiteHandTrackerFrame))]

    global _niteHandTrackerFrameAddRef
    _niteHandTrackerFrameAddRef = dll.niteHandTrackerFrameAddRef
    _niteHandTrackerFrameAddRef.restype  = NiteStatus
    _niteHandTrackerFrameAddRef.argtypes = [NiteHandTrackerHandle, ctypes.POINTER(NiteHandTrackerFrame)]

    global _niteHandTrackerFrameRelease
    _niteHandTrackerFrameRelease = dll.niteHandTrackerFrameRelease
    _niteHandTrackerFrameRelease.restype  = NiteStatus
    _niteHandTrackerFrameRelease.argtypes = [NiteHandTrackerHandle, ctypes.POINTER(NiteHandTrackerFrame)]

    global _niteStartGestureDetection
    _niteStartGestureDetection = dll.niteStartGestureDetection
    _niteStartGestureDetection.restype  = NiteStatus
    _niteStartGestureDetection.argtypes = [NiteHandTrackerHandle, NiteGestureType]

    global _niteStopGestureDetection
    _niteStopGestureDetection = dll.niteStopGestureDetection
    _niteStopGestureDetection.restype  = None
    _niteStopGestureDetection.argtypes = [NiteHandTrackerHandle, NiteGestureType]

    global _niteStopAllGestureDetection
    _niteStopAllGestureDetection = dll.niteStopAllGestureDetection
    _niteStopAllGestureDetection.restype  = None
    _niteStopAllGestureDetection.argtypes = [NiteHandTrackerHandle]

    global _niteConvertJointCoordinatesToDepth
    _niteConvertJointCoordinatesToDepth = dll.niteConvertJointCoordinatesToDepth
    _niteConvertJointCoordinatesToDepth.restype  = NiteStatus
    _niteConvertJointCoordinatesToDepth.argtypes = [NiteUserTrackerHandle, ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float)]

    global _niteConvertDepthCoordinatesToJoint
    _niteConvertDepthCoordinatesToJoint = dll.niteConvertDepthCoordinatesToJoint
    _niteConvertDepthCoordinatesToJoint.restype  = NiteStatus
    _niteConvertDepthCoordinatesToJoint.argtypes = [NiteUserTrackerHandle, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float)]

    global _niteConvertHandCoordinatesToDepth
    _niteConvertHandCoordinatesToDepth = dll.niteConvertHandCoordinatesToDepth
    _niteConvertHandCoordinatesToDepth.restype  = NiteStatus
    _niteConvertHandCoordinatesToDepth.argtypes = [NiteHandTrackerHandle, ctypes.c_float, ctypes.c_float, ctypes.c_float, ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float)]

    global _niteConvertDepthCoordinatesToHand
    _niteConvertDepthCoordinatesToHand = dll.niteConvertDepthCoordinatesToHand
    _niteConvertDepthCoordinatesToHand.restype  = NiteStatus
    _niteConvertDepthCoordinatesToHand.argtypes = [NiteHandTrackerHandle, ctypes.c_int, ctypes.c_int, ctypes.c_int, ctypes.POINTER(ctypes.c_float), ctypes.POINTER(ctypes.c_float)]

    global _niteInitializeUserTracker
    _niteInitializeUserTracker = dll.niteInitializeUserTracker
    _niteInitializeUserTracker.restype  = NiteStatus
    _niteInitializeUserTracker.argtypes = [ctypes.POINTER(NiteUserTrackerHandle)]

    global _niteInitializeUserTrackerByDevice
    _niteInitializeUserTrackerByDevice = dll.niteInitializeUserTrackerByDevice
    _niteInitializeUserTrackerByDevice.restype  = NiteStatus
    _niteInitializeUserTrackerByDevice.argtypes = [ctypes.c_void_p, ctypes.POINTER(NiteUserTrackerHandle)]

    global _niteInitializeHandTracker
    _niteInitializeHandTracker = dll.niteInitializeHandTracker
    _niteInitializeHandTracker.restype  = NiteStatus
    _niteInitializeHandTracker.argtypes = [ctypes.POINTER(NiteHandTrackerHandle)]

    global _niteInitializeHandTrackerByDevice
    _niteInitializeHandTrackerByDevice = dll.niteInitializeHandTrackerByDevice
    _niteInitializeHandTrackerByDevice.restype  = NiteStatus
    _niteInitializeHandTrackerByDevice.argtypes = [ctypes.c_void_p, ctypes.POINTER(NiteHandTrackerHandle)]

    _the_dll = dll

import functools
from primesense.utils import NiteError

def nite_call(func):
    @functools.wraps(func)
    def wrapper(*args):
        res = func(*args)
        if res != NiteStatus.NITE_STATUS_OK:
            raise NiteError(res)
        return res

    return wrapper

@nite_call
def niteInitialize():
    '''NiteStatus niteInitialize()'''
    return _niteInitialize()

def niteShutdown():
    '''void niteShutdown()'''
    return _niteShutdown()

def niteGetVersion():
    '''NiteVersion niteGetVersion()'''
    return _niteGetVersion()

@nite_call
def niteShutdownUserTracker(userTracker):
    '''NiteStatus niteShutdownUserTracker(NiteUserTrackerHandle userTracker)'''
    return _niteShutdownUserTracker(userTracker)

@nite_call
def niteStartSkeletonTracking(userTracker, id):
    '''NiteStatus niteStartSkeletonTracking(NiteUserTrackerHandle userTracker, NiteUserId id)'''
    return _niteStartSkeletonTracking(userTracker, id)

def niteStopSkeletonTracking(userTracker, id):
    '''void niteStopSkeletonTracking(NiteUserTrackerHandle userTracker, NiteUserId id)'''
    return _niteStopSkeletonTracking(userTracker, id)

def niteIsSkeletonTracking(userTracker, id):
    '''bool niteIsSkeletonTracking(NiteUserTrackerHandle userTracker, NiteUserId id)'''
    return _niteIsSkeletonTracking(userTracker, id)

@nite_call
def niteSetSkeletonSmoothing(userTracker, factor):
    '''NiteStatus niteSetSkeletonSmoothing(NiteUserTrackerHandle userTracker, float factor)'''
    return _niteSetSkeletonSmoothing(userTracker, factor)

@nite_call
def niteGetSkeletonSmoothing(userTracker, pFactor):
    '''NiteStatus niteGetSkeletonSmoothing(NiteUserTrackerHandle userTracker, float* pFactor)'''
    return _niteGetSkeletonSmoothing(userTracker, pFactor)

@nite_call
def niteStartPoseDetection(userTracker, id, poseType):
    '''NiteStatus niteStartPoseDetection(NiteUserTrackerHandle userTracker, NiteUserId id, NitePoseType poseType)'''
    return _niteStartPoseDetection(userTracker, id, poseType)

def niteStopPoseDetection(userTracker, id, poseType):
    '''void niteStopPoseDetection(NiteUserTrackerHandle userTracker, NiteUserId id, NitePoseType poseType)'''
    return _niteStopPoseDetection(userTracker, id, poseType)

def niteStopAllPoseDetection(userTracker, id):
    '''void niteStopAllPoseDetection(NiteUserTrackerHandle userTracker, NiteUserId id)'''
    return _niteStopAllPoseDetection(userTracker, id)

@nite_call
def niteRegisterUserTrackerCallbacks(userTracker, pCallbacks, pCookie):
    '''NiteStatus niteRegisterUserTrackerCallbacks(NiteUserTrackerHandle userTracker, NiteUserTrackerCallbacks* pCallbacks, void* pCookie)'''
    return _niteRegisterUserTrackerCallbacks(userTracker, pCallbacks, pCookie)

def niteUnregisterUserTrackerCallbacks(userTracker, pCallbacks):
    '''void niteUnregisterUserTrackerCallbacks(NiteUserTrackerHandle userTracker, NiteUserTrackerCallbacks* pCallbacks)'''
    return _niteUnregisterUserTrackerCallbacks(userTracker, pCallbacks)

@nite_call
def niteReadUserTrackerFrame(userTracker, pUserTrackerFrame):
    '''NiteStatus niteReadUserTrackerFrame(NiteUserTrackerHandle userTracker, NiteUserTrackerFrame** pUserTrackerFrame)'''
    return _niteReadUserTrackerFrame(userTracker, pUserTrackerFrame)

@nite_call
def niteUserTrackerFrameAddRef(userTracker, pUserTrackerFrame):
    '''NiteStatus niteUserTrackerFrameAddRef(NiteUserTrackerHandle userTracker, NiteUserTrackerFrame* pUserTrackerFrame)'''
    return _niteUserTrackerFrameAddRef(userTracker, pUserTrackerFrame)

@nite_call
def niteUserTrackerFrameRelease(userTracker, pUserTrackerFrame):
    '''NiteStatus niteUserTrackerFrameRelease(NiteUserTrackerHandle userTracker, NiteUserTrackerFrame* pUserTrackerFrame)'''
    return _niteUserTrackerFrameRelease(userTracker, pUserTrackerFrame)

@nite_call
def niteDumpUserTrackerCalibrationDataToFile(userTracker, id, targetFile):
    '''NiteStatus niteDumpUserTrackerCalibrationDataToFile(NiteUserTrackerHandle userTracker, NiteUserId id, char* targetFile)'''
    return _niteDumpUserTrackerCalibrationDataToFile(userTracker, id, targetFile)

@nite_call
def niteShutdownHandTracker(handTracker):
    '''NiteStatus niteShutdownHandTracker(NiteHandTrackerHandle handTracker)'''
    return _niteShutdownHandTracker(handTracker)

@nite_call
def niteStartHandTracking(handTracker, pPosition, pNewHandId):
    '''NiteStatus niteStartHandTracking(NiteHandTrackerHandle handTracker, NitePoint3f* pPosition, NiteHandId* pNewHandId)'''
    return _niteStartHandTracking(handTracker, pPosition, pNewHandId)

def niteStopHandTracking(handTracker, id):
    '''void niteStopHandTracking(NiteHandTrackerHandle handTracker, NiteHandId id)'''
    return _niteStopHandTracking(handTracker, id)

def niteStopAllHandTracking(handTracker):
    '''void niteStopAllHandTracking(NiteHandTrackerHandle handTracker)'''
    return _niteStopAllHandTracking(handTracker)

@nite_call
def niteSetHandSmoothingFactor(handTracker, factor):
    '''NiteStatus niteSetHandSmoothingFactor(NiteHandTrackerHandle handTracker, float factor)'''
    return _niteSetHandSmoothingFactor(handTracker, factor)

@nite_call
def niteGetHandSmoothingFactor(handTracker, pFactor):
    '''NiteStatus niteGetHandSmoothingFactor(NiteHandTrackerHandle handTracker, float* pFactor)'''
    return _niteGetHandSmoothingFactor(handTracker, pFactor)

@nite_call
def niteRegisterHandTrackerCallbacks(handTracker, pCallbacks, pCookie):
    '''NiteStatus niteRegisterHandTrackerCallbacks(NiteHandTrackerHandle handTracker, NiteHandTrackerCallbacks* pCallbacks, void* pCookie)'''
    return _niteRegisterHandTrackerCallbacks(handTracker, pCallbacks, pCookie)

def niteUnregisterHandTrackerCallbacks(handTracker, pCallbacks):
    '''void niteUnregisterHandTrackerCallbacks(NiteHandTrackerHandle handTracker, NiteHandTrackerCallbacks* pCallbacks)'''
    return _niteUnregisterHandTrackerCallbacks(handTracker, pCallbacks)

@nite_call
def niteReadHandTrackerFrame(handTracker, pHandTrackerFrame):
    '''NiteStatus niteReadHandTrackerFrame(NiteHandTrackerHandle handTracker, NiteHandTrackerFrame** pHandTrackerFrame)'''
    return _niteReadHandTrackerFrame(handTracker, pHandTrackerFrame)

@nite_call
def niteHandTrackerFrameAddRef(handTracker, pHandTrackerFrame):
    '''NiteStatus niteHandTrackerFrameAddRef(NiteHandTrackerHandle handTracker, NiteHandTrackerFrame* pHandTrackerFrame)'''
    return _niteHandTrackerFrameAddRef(handTracker, pHandTrackerFrame)

@nite_call
def niteHandTrackerFrameRelease(handTracker, pHandTrackerFrame):
    '''NiteStatus niteHandTrackerFrameRelease(NiteHandTrackerHandle handTracker, NiteHandTrackerFrame* pHandTrackerFrame)'''
    return _niteHandTrackerFrameRelease(handTracker, pHandTrackerFrame)

@nite_call
def niteStartGestureDetection(handTracker, type):
    '''NiteStatus niteStartGestureDetection(NiteHandTrackerHandle handTracker, NiteGestureType type)'''
    return _niteStartGestureDetection(handTracker, type)

def niteStopGestureDetection(handTracker, type):
    '''void niteStopGestureDetection(NiteHandTrackerHandle handTracker, NiteGestureType type)'''
    return _niteStopGestureDetection(handTracker, type)

def niteStopAllGestureDetection(handTracker):
    '''void niteStopAllGestureDetection(NiteHandTrackerHandle handTracker)'''
    return _niteStopAllGestureDetection(handTracker)

@nite_call
def niteConvertJointCoordinatesToDepth(userTracker, x, y, z, pX, pY):
    '''NiteStatus niteConvertJointCoordinatesToDepth(NiteUserTrackerHandle userTracker, float x, float y, float z, float* pX, float* pY)'''
    return _niteConvertJointCoordinatesToDepth(userTracker, x, y, z, pX, pY)

@nite_call
def niteConvertDepthCoordinatesToJoint(userTracker, x, y, z, pX, pY):
    '''NiteStatus niteConvertDepthCoordinatesToJoint(NiteUserTrackerHandle userTracker, int x, int y, int z, float* pX, float* pY)'''
    return _niteConvertDepthCoordinatesToJoint(userTracker, x, y, z, pX, pY)

@nite_call
def niteConvertHandCoordinatesToDepth(handTracker, x, y, z, pX, pY):
    '''NiteStatus niteConvertHandCoordinatesToDepth(NiteHandTrackerHandle handTracker, float x, float y, float z, float* pX, float* pY)'''
    return _niteConvertHandCoordinatesToDepth(handTracker, x, y, z, pX, pY)

@nite_call
def niteConvertDepthCoordinatesToHand(handTracker, x, y, z, pX, pY):
    '''NiteStatus niteConvertDepthCoordinatesToHand(NiteHandTrackerHandle handTracker, int x, int y, int z, float* pX, float* pY)'''
    return _niteConvertDepthCoordinatesToHand(handTracker, x, y, z, pX, pY)

@nite_call
def niteInitializeUserTracker(pUserTracker):
    '''NiteStatus niteInitializeUserTracker(NiteUserTrackerHandle* pUserTracker)'''
    return _niteInitializeUserTracker(pUserTracker)

@nite_call
def niteInitializeUserTrackerByDevice(pDevice, pUserTracker):
    '''NiteStatus niteInitializeUserTrackerByDevice(void* pDevice, NiteUserTrackerHandle* pUserTracker)'''
    return _niteInitializeUserTrackerByDevice(pDevice, pUserTracker)

@nite_call
def niteInitializeHandTracker(pHandTracker):
    '''NiteStatus niteInitializeHandTracker(NiteHandTrackerHandle* pHandTracker)'''
    return _niteInitializeHandTracker(pHandTracker)

@nite_call
def niteInitializeHandTrackerByDevice(pDevice, pHandTracker):
    '''NiteStatus niteInitializeHandTrackerByDevice(void* pDevice, NiteHandTrackerHandle* pHandTracker)'''
    return _niteInitializeHandTrackerByDevice(pDevice, pHandTracker)

all_types = [
    OniStatus,
    OniSensorType,
    OniPixelFormat,
    OniDeviceState,
    OniImageRegistrationMode,
    _anon_enum_5,
    OniBool,
    OniCallbackHandleImpl,
    OniCallbackHandle,
    OniVersion,
    OniHardwareVersion,
    OniVideoMode,
    OniSensorInfo,
    OniDeviceInfo,
    _OniDevice,
    OniDeviceHandle,
    _OniStream,
    OniStreamHandle,
    _OniRecorder,
    OniRecorderHandle,
    OniFrame,
    OniNewFrameCallback,
    OniGeneralCallback,
    OniDeviceInfoCallback,
    OniDeviceStateCallback,
    OniFrameAllocBufferCallback,
    OniFrameFreeBufferCallback,
    OniDeviceCallbacks,
    OniCropping,
    OniDepthPixel,
    OniGrayscale16Pixel,
    OniGrayscale8Pixel,
    OniRGB888Pixel,
    OniYUV422DoublePixel,
    OniSeek,
    NiteJointType,
    NiteSkeletonState,
    NiteUserState,
    NiteStatus,
    NitePoseType,
    NitePoseState,
    NiteGestureType,
    NiteGestureState,
    NiteHandState,
    NiteUserId,
    NiteUserTracker,
    NiteUserTrackerHandle,
    NitePoint3f,
    NiteQuaternion,
    NiteSkeletonJoint,
    NiteBoundingBox,
    NitePoseData,
    NiteSkeleton,
    NiteUserData,
    NiteUserMap,
    NitePlane,
    NiteUserTrackerFrame,
    NiteUserTrackerCallbacks,
    NiteHandId,
    NiteHandData,
    NiteGestureData,
    NiteHandTrackerFrame,
    NiteHandTrackerCallbacks,
    NiteVersion,
    NiteHandTracker,
    NiteHandTrackerHandle,
]

all_funcs = [
    niteInitialize,
    niteShutdown,
    niteGetVersion,
    niteShutdownUserTracker,
    niteStartSkeletonTracking,
    niteStopSkeletonTracking,
    niteIsSkeletonTracking,
    niteSetSkeletonSmoothing,
    niteGetSkeletonSmoothing,
    niteStartPoseDetection,
    niteStopPoseDetection,
    niteStopAllPoseDetection,
    niteRegisterUserTrackerCallbacks,
    niteUnregisterUserTrackerCallbacks,
    niteReadUserTrackerFrame,
    niteUserTrackerFrameAddRef,
    niteUserTrackerFrameRelease,
    niteDumpUserTrackerCalibrationDataToFile,
    niteShutdownHandTracker,
    niteStartHandTracking,
    niteStopHandTracking,
    niteStopAllHandTracking,
    niteSetHandSmoothingFactor,
    niteGetHandSmoothingFactor,
    niteRegisterHandTrackerCallbacks,
    niteUnregisterHandTrackerCallbacks,
    niteReadHandTrackerFrame,
    niteHandTrackerFrameAddRef,
    niteHandTrackerFrameRelease,
    niteStartGestureDetection,
    niteStopGestureDetection,
    niteStopAllGestureDetection,
    niteConvertJointCoordinatesToDepth,
    niteConvertDepthCoordinatesToJoint,
    niteConvertHandCoordinatesToDepth,
    niteConvertDepthCoordinatesToHand,
    niteInitializeUserTracker,
    niteInitializeUserTrackerByDevice,
    niteInitializeHandTracker,
    niteInitializeHandTrackerByDevice,
]



#!/usr/bin/env python
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

setup(name = "primesense",
    version = "2.2.0.30-4",
    description = "OpenNI2 and NiTE2 python bindings",
    author = "PrimeSense Inc",
    author_email = "primesense.com",
    license = "MIT",
    url = "http://www.openni.org/",
    packages = ["primesense"],
    platforms = ["POSIX", "Windows"],
    provides = ["primesense"],
    keywords = "PrimeSense, OpenNI, OpenNI2, Natural Interaction, NiTE, NiTE2",
    long_description = """\
Official `PrimeSense <http://www.primesense.com/>`_ Python-bindings for `OpenNI2 <http://www.openni.org/>`_
and `NiTE2 <http://www.openni.org/files/nite/>`_.

This package provides only the Python bindings; be sure to install OpenNI (and optionally NiTE) first.

Example::
    
    from primesense import openni2
    
    openni2.initialize()     # can also accept the path of the OpenNI redistribution
    
    dev = openni2.Device.open_any()
    print dev.get_sensor_info()
    
    depth_stream = dev.create_depth_stream()
    depth_stream.start()
    frame = depth_stream.read_frame()
    frame_data = frame.get_buffer_as_uint16()
    depth_stream.stop()
    
    openni2.unload()


.. note:: Refer to the C API for documentation

""",
    classifiers = [
        "Development Status :: 4 - Beta",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
    ],
)

